#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "bb.h"
#include "pool.h"

/* calcule et renvoie la borne sup sur sp
   on suppose qu'il n y a pas de variables libres de poids > B(sp) 
   si la solution de la relaxation est entire :
     - on modifie sp en mettant dedans cette solution
     - on met -1 dans frac_var
   sinon
     - on met l'indice de la variable non-entire dans frac_var */

int calcul_maj(kp_t *kp, sp_t sp, int *frac_var)
{  int reste, score, i, j;
   int *p, *q, *r;
   
   reste = B(sp);
   score = C(sp);
   for (i = 0; i < NLIB(sp); i++)
   {  j = LIB(sp)[i];
      if (kp->a[j] > reste) break;
      reste -= kp->a[j];
      score += kp->c[j];
   }
   
   if (i == NLIB(sp) || reste == 0)
   {  p = LIB(sp) + i;
      q = FIX1(sp);
      r = q + NFIX1(sp);
      C(sp) = score;
      B(sp) = reste;
      *frac_var = -1;
      NFIX1(sp) += i;
      NLIB(sp) = 0;
      if (p != q)
         while (q != r) *(p++) = *(q++);
      return score;
   }
   
   *frac_var = i;
   score += kp->c[j] * reste / kp->a[j];
   return score; 
}


/* sparation d'un sous-problme
   au dbut :
     - sp contient le sous-problme  sparer
     - frac_var cotient l'indice de la variable non-entire
    la fin :
     - sp  contient le sous-problme dans lequel frac_var est fixe  0
     - sp1 contient le sous-problme dans lequel frac_var est fixe  1 */

void separation(kp_t *kp, sp_t sp, sp_t sp1, int frac_var)
{  int i, j, jf;

   /* fixation  1 */
   jf = LIB(sp)[frac_var];
   C(sp1) = C(sp) + kp->c[jf];
   B(sp1) = B(sp) - kp->a[jf];
   NLIB(sp1) = 0;
   for (i = 0; i < NLIB(sp); i++)
   {  j = LIB(sp)[i];
      /* on met  0 les vriables de poids > B(sp1) */
      if (j == jf || kp->a[j] > B(sp1)) continue;
      LIB(sp1)[NLIB(sp1)] = j;
      NLIB(sp1)++;
   }
   NFIX1(sp1) = NFIX1(sp);
   for (i = 0; i < NFIX1(sp); i++)
      FIX1(sp1)[i] = FIX1(sp)[i];
   FIX1(sp1)[NFIX1(sp1)] = jf;
   NFIX1(sp1)++;

   /* fixation  0 */
   NLIB(sp)--;
   for (i = frac_var; i < NLIB(sp); i++)
      LIB(sp)[i] = LIB(sp)[i + 1];
   if (NFIX1(sp) > 0)
      FIX1(sp)[0] = FIX1(sp)[NFIX1(sp)];
}


void branch_and_bound(kp_t *kp, sp_t initial_pb, sp_t best_sol)
{  pool_t *pool;
   sp_t sp, sp1;
   int ub, frac_var;
   int nodes = 0;

   printf("RECORD INITIAL : %d\n\n", C(best_sol));
   printf("     NOEUD  RESTANTS BORNE_SUP    REDORD     ACTION\n");
   printf("-----------------------------------------------------------\n");
   
   pool = p_init();
   p_add(pool, initial_pb);
   while(!p_isempty(pool))
   {  sp = p_extract(pool);
      ub = calcul_maj(kp, sp, &frac_var);
      printf("%10d%10d%10d%10d     ", ++nodes, pool->n, ub, C(best_sol));
      if (ub <= C(best_sol)) 
      {  printf("ELAGAGE\n");
         free(sp);
      }
      else if (frac_var == -1)
      {  printf("NOUVEAU RECORD\n");
         memcpy(best_sol, sp, (4 + NFIX1(sp)) * sizeof(int));
         free(sp);
      }
      else
      {  printf("SEPARATION\n");
         sp1 = malloc((4 + NLIB(sp) + NFIX1(sp)) * sizeof(int)); 
         separation(kp, sp, sp1, frac_var);
         p_add(pool, sp);
	 p_add(pool, sp1);
      }
   }
   
   p_free(pool);
}
