#ifndef KP_H
#define KP_H


/* structure utilise pour reprsenter le problme de sac--dos */
typedef struct
{  int n;  /* nombre d'objets */
   int b;  /* capacit */
   int *c; /* cots */
   int *a; /* poids */
} kp_t;


/* un sous-problme est represet par un tableau d'eniers sp
   sp[0] - cot de variables fixes  1
   sp[1] - capacit restante
   sp[2] - nombre de variables libres (nlib)
   sp[3] - nombre de variables fixes  1 (nfix1)
   sp[4] ... sp[nlib + 3] - liste de variables libres (en ordre dcroissant
      de c[j] / a[j]
   sp[nlib + 4] ... sp[nlib + nfree + 3] - liste de variables fixes  1
   la mme structure peut reprsenter une solution (si nlib = 0)
*/
typedef int *sp_t;


/* les macros suivants sont utiliss pour accder facilement aux
   lments d'un sous-problme */

/* cot des variables fixes  1 */
#define C(sp) ((sp)[0])

/* capacit restante */
#define B(sp) ((sp)[1])

/* nombre de variables libres */
#define NLIB(sp) ((sp)[2])

/* nombre de variables fixes  1 */
#define NFIX1(sp) ((sp)[3])

/* tableau de variables libres */
#define LIB(sp) ((sp) + 4)

/* tableau de variables fixes  1 */
#define FIX1(sp) ((sp) + 4 + (sp)[2])


/* lit les donnes  partir du fichier spcifi
   cre et renvoie la structure kp_t correspondante */
kp_t *lire_kp(char *fichier);


/* nettoyage de la structure kp_t */
void free_kp(kp_t *kp);


/* trie les objets et ordre dcroissant de c[j] / a[j],
   met dans sp le problme initial
   limine les variables pour lesquelles a[j] > b */
void trier_objets(kp_t *kp, sp_t sp);


/* recherche de solution ralisable
   applique l'algorithme sur sp
   met la solution trouve dans sol */
void sol_glouton(kp_t *kp, sp_t sp, sp_t sol);

#endif
